//Copyright (C) 2004, Brian Enigma <enigma at netninja.com>
//This file is part of Tasty Safari.
//
//Tasty Safari is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//Tasty Safari is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with Foobar; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package org.ninjasoft.tastysafari.logic;

import java.util.*;

import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.ninjasoft.tastysafari.data.*;

/**
 */
public class SafariBookmarkMerger {
    String username = "";
    Hashtable usernames = new Hashtable();
    
    public void setUsername(String s) {
        this.username = s;
    }
    
    public void merge(Document doc, 
                      Vector entries, 
                      PostEntry[] mostRecentEntries, 
                      StatusListener listener,
                      boolean includeHome,
                      boolean includeMy,
                      boolean includeInbox) throws TransformerException, UnexpectedXmlFileFormatException {
        createDatastructure(entries);
        Object[] usernameList = usernames.keySet().toArray();
        Arrays.sort(usernameList);
        // Put yourself first in the list
        for (int i=1; i<usernameList.length; i++) {
            // Found self, push to front of list
            if (((String) usernameList[i]).equalsIgnoreCase(this.username)) {
                Object self = usernameList[i];
                for (int j=i; j>0; j--) {
                    usernameList[j] = usernameList[j-1];
                }
                usernameList[0] = self;
            }
        }
        Element bookmarkBarNode = SafariBookmarkEditor.locateBookmarkBarNode(doc);
        Element deliciousFolder = SafariBookmarkEditor.locateCreateSubFolder(bookmarkBarNode, "del.icio.us");
        SafariBookmarkEditor.deleteChildren(deliciousFolder);
        // Create the pre-canned stuff
        SafariBookmarkEditor.addLink(deliciousFolder, "Add Bookmark", "javascript:q=location.href;p=document.title;void(open('http://del.icio.us/" + this.username + "?noui=yes&jump=close&url='+escape(q)+'&title='+escape(p),'delicious',%20'toolbar=no,width=700,height=250'));");
        if (includeHome)
            SafariBookmarkEditor.addLink(deliciousFolder, "Delicious Home", "http://del.icio.us/");
        if (includeMy)
        	SafariBookmarkEditor.addLink(deliciousFolder, "My Delicious", "http://del.icio.us/" + this.username);
        if (includeInbox)
            SafariBookmarkEditor.addLink(deliciousFolder, "My Delicious Inbox", "http://del.icio.us/inbox/" + this.username);
        // Create the subscribed stuff
        for (int userNumber = 0; userNumber < usernameList.length; userNumber++) {
            listener.setStatus((String) usernameList[userNumber]);
            Hashtable categories = (Hashtable) usernames.get(usernameList[userNumber]);
            Object[] categoryList = categories.keySet().toArray();
            Arrays.sort(categoryList);
            for (int categoryNumber = 0; categoryNumber < categoryList.length; categoryNumber++) {
                Hashtable entryTable = (Hashtable) categories.get(categoryList[categoryNumber]);
                Object[] entryList = entryTable.values().toArray();
                Arrays.sort(entryList, new Comparator() {
                    public int compare(Object o1, Object o2) {
                        String s1 = ((PostEntry) o1).getDescription();
                        String s2 = ((PostEntry) o2).getDescription();
                        return s1.compareTo(s2);
                    }
                    public boolean equals(Object o) {
                        return o.getClass().getName().equals(this.getClass().getName());
                    }
                });
                PostEntry[] sortedEntries = new PostEntry[entryList.length];
                for (int i=0; i<entryList.length; i++)
                    sortedEntries[i] = (PostEntry) entryList[i];
                // If there is only one user in the list (and it's yourself), display only categories and not username
                if ((usernameList.length == 1) && ((String) usernameList[0]).equalsIgnoreCase(this.username)) {
                    SafariBookmarkEditor.graftBookmarks(deliciousFolder, 
                            null, 
                            (String) categoryList[categoryNumber], 
                            sortedEntries);
                }else{
                    SafariBookmarkEditor.graftBookmarks(deliciousFolder, 
                                                        (String) usernameList[userNumber], 
                                                        (String) categoryList[categoryNumber], 
                                                        sortedEntries);
                }
            }
        }
        // If there are any most recent entries, append them...
        if ((mostRecentEntries != null) && (mostRecentEntries.length > 0)) {
            if (mostRecentEntries.length > 10) {
                PostEntry[] tempEntries = new PostEntry[10];
                for (int i=0; i<10; i++)
                    tempEntries[i] = mostRecentEntries[mostRecentEntries.length - 10 + i];
                mostRecentEntries = tempEntries;
            }
        	SafariBookmarkEditor.appendBookmarkToFolder(deliciousFolder, SafariBookmarkEditor.createMostRecent(deliciousFolder.getOwnerDocument(), mostRecentEntries.length));
        	for (int i=0; i<mostRecentEntries.length; i++) {
                mostRecentEntries[i].setDescription("   " + mostRecentEntries[i].getDescription());
        		SafariBookmarkEditor.appendBookmarkToFolder(deliciousFolder, mostRecentEntries[i]);
            }
        }
        listener.setStatus("Processed " + usernameList.length + " user(s)");
    }
    
    private void createDatastructure(Vector entries) {
        for (Iterator it = entries.iterator(); it.hasNext(); ) {
            PostEntry[] posts = (PostEntry[]) it.next();
            for (int i=0; i<posts.length; i++) {
                PostEntry post = posts[i];
            	Hashtable categories = getCreate(post.getUser(), this.usernames);
                String[] tags = post.getTags();
                for (int j=0; j<tags.length; j++) {
                    Hashtable category = getCreate(tags[j], categories);
                    category.put(post.getDescription(), post);
                }
            }
        }
    }
    
    private Hashtable getCreate(String name, Hashtable table) {
        Hashtable result = (Hashtable) table.get(name);
        if (result == null) {
            result = new Hashtable();
            table.put(name, result);
        }
        return result;
    }
}
