// Copyright (C) 2004, Brian Enigma <enigma at netninja.com>
// This file is part of Tasty Safari.
//
// Tasty Safari is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Tasty Safari is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package org.ninjasoft.tastysafari.swingui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.w3c.dom.Document;
import javax.swing.border.*;

import com.apple.eawt.*;

import org.ninjasoft.tastysafari.logic.*;
import org.ninjasoft.tastysafari.data.PostEntry;
import org.ninjasoft.tastysafari.data.ApplicationProperties;

public class MainWindow extends JFrame implements StatusListener, Runnable, KeyListener {
    private String version = "";
    private Actions actions = new Actions();
    private HighlightFocuser highlightFocuser = new HighlightFocuser();
    private JTextField urlText = new JTextField("del.icio.us");
    private JTextField usernameText = new JTextField("");
    private JTextField passwordText = new JPasswordField();
    private JCheckBox checkboxRecent = new JCheckBox("Individual recent item links");
    private JCheckBox checkboxDeliciousHome = new JCheckBox("Delicious homepage");
    private JCheckBox checkboxMyDelicious = new JCheckBox("My Delicious");
    private JCheckBox checkboxDeliciousInbox = new JCheckBox("My Inbox");
    private JRadioButton radioSubscriptions = new JRadioButton("subscriptions");
    private JRadioButton radioBookmarks = new JRadioButton("bookmarks");
    private JButton okButton = new JButton("Proceed");
    private JButton cancelButton = new JButton("Cancel");
    private JEditorPane stepText = new JEditorPane("text/html", "");
    private int currentStep = -1;
    private static String[] STEP_DESCRIPTIONS = new String[] {
            "Read existing Safari bookmark file",
            "Download bookmarks",
            "Process data",
            "Backup existing Safari bookmark file",
            "Write new Safari bookmark file"};
    private String[] stepDetails = new String[5];
    private SafariBookmarkEditor bookmarkEditor = null;
    private ApplicationProperties properties = null;
    private PreviewPanel previewPanel = new PreviewPanel();
    
    public MainWindow() {
        super();
        bookmarkEditor = new SafariBookmarkEditor();
        resetStepDetails();
        initializeVersion();
        initializeGui();
        initializeEvents();
        initializeApple();
        initializePreferences();
        urlText.setEditable(false);
        usernameText.setSelectionStart(0);
        usernameText.setSelectionEnd(usernameText.getText().length());
        usernameText.requestFocus();
    }
    
    private void initializeVersion() {
        InputStream in = this.getClass().getResourceAsStream("/org/ninjasoft/tastysafari/version.properties");
        if (in == null)
            return;
        Properties versionProperties = new Properties();
        try{
        	versionProperties.load(in);
        	in.close();
        }catch(Exception e) {}
        String versionTemp = versionProperties.getProperty("version", "");
        if ((versionTemp != null) && (versionTemp.length() > 0) && !versionTemp.equals("@VERSION@"))
            this.version = versionTemp;
    }
    
    private void initializePreferences() {
        properties = null;
        try{
            properties = new ApplicationProperties();
            usernameText.setText(properties.getProperty("username", System.getProperty("user.name")));
            passwordText.setText(properties.getProperty("password", ""));
            checkboxRecent.setSelected(properties.getProperty("recent", "false").equalsIgnoreCase("true"));
            checkboxDeliciousHome.setSelected(properties.getProperty("home", "true").equalsIgnoreCase("true"));
            checkboxMyDelicious.setSelected(properties.getProperty("my", "true").equalsIgnoreCase("true"));
            checkboxDeliciousInbox.setSelected(properties.getProperty("inbox", "true").equalsIgnoreCase("true"));
            if (properties.getProperty("download", "subscriptions").equalsIgnoreCase("subscriptions"))
                radioSubscriptions.setSelected(true);
            else
                radioBookmarks.setSelected(true);
        }catch(IOException e){
            JOptionPane.showMessageDialog(this, "Problem opening program preferences\n" + e.toString(), "Unexpected Error", JOptionPane.ERROR_MESSAGE);
        }
    }
    
    public void dispose() {
        endProgram();
        super.dispose();
    }

    private void endProgram() {
        this.setVisible(false);
        try{
            properties.setProperty("username", usernameText.getText());
            properties.setProperty("password", passwordText.getText());
            properties.setProperty("recent", checkboxRecent.isSelected() ? "true" : "false");
            properties.setProperty("home", checkboxDeliciousHome.isSelected() ? "true" : "false");
            properties.setProperty("my", checkboxMyDelicious.isSelected() ? "true" : "false");
            properties.setProperty("inbox", checkboxDeliciousInbox.isSelected() ? "true" : "false");
            properties.setProperty("download", radioSubscriptions.isSelected() ? "subscriptions" : "bookmarks");
            if (properties != null)
            	properties.save();
        }catch(IOException e){
            e.printStackTrace();
        }
        System.exit(0);
    }
    
    private void initializeApple() {
        System.setProperty("com.apple.mrj.application.apple.menu.about.name", "Tasty Safari");
        System.setProperty("com.apple.laf.useScreenMenuBar", "true");
        System.setProperty("apple.awt.brushMetalLook", "true");
        Application application = Application.getApplication();
        application.addApplicationListener(new ApplicationAdapter() {
            public void handleQuit(ApplicationEvent event) {
                endProgram();
            }
        });
        application.setEnabledAboutMenu(false);
        application.setEnabledPreferencesMenu(false);
        
        
    }
    
    private void resetStepDetails() {
        currentStep = -1;
        stepDetails[0] = bookmarkEditor.getBookmarkFilename();
        stepDetails[1] = "0 items downloaded";
        stepDetails[2] = "";
        stepDetails[3] = "There are currently " + bookmarkEditor.getBackupCount() + " previous backup" + (bookmarkEditor.getBackupCount() == 1 ? "" : "s");
        stepDetails[4] = "0 bytes written";
        stepText.setText(buildStatusString(currentStep));
    }
    
    private void initializeGui() {
        this.setTitle("Tasty Safari " + this.version);
        this.setSize(652, 500);
        stepText.setOpaque(false);
        Dimension windowSize = this.getSize();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        this.setLocation(screenSize.width/2 - windowSize.width/2, screenSize.height/2 - windowSize.height/2);
        this.setResizable(false);
        Container pane = this.getContentPane();
        pane.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        // Server line
        c.gridx = 0;
        c.gridy = 0;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.EAST;
        c.weightx = 0;
        c.weighty = 0;
        c.gridwidth = 1;
        c.insets = new Insets(2, 15, 2, 2);
        pane.add(new Label("Server"), c);
        c.gridx++;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.WEST;
        c.weightx = 1;
        c.weighty = 0;
        c.gridwidth = 3;
        c.insets = new Insets(2, 2, 2, 15);
        pane.add(urlText, c);
        // Username line
        c.gridx = 0;
        c.gridy++;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.EAST;
        c.weightx = 0;
        c.weighty = 0;
        c.gridwidth = 1;
        c.insets = new Insets(2, 15, 2, 2);
        pane.add(new Label("Username"), c);
        c.gridx++;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.WEST;
        c.weightx = 1;
        c.weighty = 0;
        c.gridwidth = 3;
        c.insets = new Insets(2, 2, 2, 15);
        pane.add(usernameText, c);
        // Password line
        c.gridx = 0;
        c.gridy++;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.EAST;
        c.weightx = 0;
        c.weighty = 0;
        c.gridwidth = 1;
        c.insets = new Insets(2, 15, 2, 2);
        pane.add(new Label("Password"), c);
        c.gridx++;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.WEST;
        c.weightx = 1;
        c.weighty = 0;
        c.gridwidth = 3;
        c.insets = new Insets(2, 2, 2, 15);
        pane.add(passwordText, c);
        // Download line
        c.gridx = 0;
        c.gridy++;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.EAST;
        c.weightx = 0;
        c.weighty = 0;
        c.gridwidth = 1;
        c.insets = new Insets(2, 15, 2, 2);
        pane.add(new Label("Download my"), c);
        c.gridx++;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.WEST;
        c.weightx = 1;
        c.weighty = 0;
        c.gridwidth = 3;
        c.insets = new Insets(2, 2, 2, 15);
        ButtonGroup group = new ButtonGroup();
        group.add(this.radioSubscriptions);
        group.add(this.radioBookmarks);
        JPanel radioPanel = new JPanel();
        radioPanel.setLayout(new BoxLayout(radioPanel, BoxLayout.X_AXIS));
        radioPanel.add(this.radioSubscriptions);
        radioPanel.add(this.radioBookmarks);
        pane.add(radioPanel, c);
        // Homepages line
        c.gridx = 0;
        c.gridy++;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.NORTHEAST;
        c.weightx = 0;
        c.weighty = 0;
        c.gridwidth = 1;
        c.insets = new Insets(2, 15, 2, 2);
        pane.add(new Label("Include bookmarks for:"), c);
        c.gridx++;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.NORTHWEST;
        c.weightx = 1;
        c.weighty = 0;
        c.gridwidth = 3;
        c.insets = new Insets(2, 2, 2, 15);
        JPanel bookmarkPanel = new JPanel();
        bookmarkPanel.setLayout(new BoxLayout(bookmarkPanel, BoxLayout.Y_AXIS));
        JCheckBox tempCheckbox = new JCheckBox("Popup bookmarking window", true);
        tempCheckbox.setSelected(true);
        tempCheckbox.setEnabled(false);
        bookmarkPanel.add(tempCheckbox);
        bookmarkPanel.add(checkboxDeliciousHome);
        bookmarkPanel.add(checkboxMyDelicious);
        bookmarkPanel.add(checkboxDeliciousInbox);
        bookmarkPanel.add(checkboxRecent);
        pane.add(bookmarkPanel, c);
        
        //Spacer
        c.gridx = 0;
        c.gridy++;
        c.fill = GridBagConstraints.BOTH;
        c.anchor = GridBagConstraints.CENTER;
        c.weightx = 2;
        c.weighty = 2;
        c.gridwidth = 4;
        c.insets = new Insets(2, 15, 2, 15);
        pane.add(stepText, c);
        // Buttons
        c.gridx = 0;
        c.gridy++;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.WEST;
        c.weightx = 0;
        c.weighty = 0;
        c.gridwidth = 1;
        c.insets = new Insets(15, 15, 15, 15);
        pane.add(cancelButton, c);
        c.gridx += 2;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.EAST;
        c.weightx = 1;
        c.weighty = 0;
        c.gridwidth = 2;
        c.insets = new Insets(15, 15, 15, 15);
        pane.add(okButton, c);
        
        // Preview
        c.gridx = 4;
        c.gridy = 0;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.NORTH;
        c.gridwidth = 1;
        c.gridheight = 6;
        c.weightx = 1;
        c.weighty = 1;
        JPanel previewTemp = new JPanel();
        previewTemp.setLayout(new BorderLayout());
        previewTemp.add(new JLabel("Preview:"), BorderLayout.NORTH);
        previewTemp.add(previewPanel, BorderLayout.CENTER);
        previewTemp.setBorder(new CompoundBorder(
                              new EtchedBorder(EtchedBorder.LOWERED),
                              new EmptyBorder(5, 5, 5, 5)));
        previewTemp.setBackground(Color.white);
        pane.add(previewTemp, c);
    }
    
    private void initializeEvents() {
        this.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        cancelButton.setEnabled(false);
        okButton.setActionCommand("okButton");
        okButton.addActionListener(actions);
        cancelButton.setActionCommand("cancelButton");
        cancelButton.addActionListener(actions);
        Component[] components = this.getContentPane().getComponents();
        // Add listeners that highlight a field when tabbed to it, as well as
        // listeners to watch for Enter/Esc
        for (int i=0; i<components.length; i++) {
            if (components[i] instanceof JTextField)
                ((JTextField) components[i]).addFocusListener(highlightFocuser);
            components[i].addKeyListener(this);
        }
        this.addKeyListener(this);
        ChangeListener buttonChangeListener = new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                updatePreview();
            }
        };
        checkboxDeliciousHome.addChangeListener(buttonChangeListener);
        checkboxMyDelicious.addChangeListener(buttonChangeListener);
        checkboxDeliciousInbox.addChangeListener(buttonChangeListener);
        checkboxRecent.addChangeListener(buttonChangeListener);
        radioSubscriptions.addChangeListener(buttonChangeListener);
        radioBookmarks.addChangeListener(buttonChangeListener);
    }
    
    private void updatePreview() {
        previewPanel.setHomeVisible(checkboxDeliciousHome.isSelected());
        previewPanel.setMyVisible(checkboxMyDelicious.isSelected());
        previewPanel.setInboxVisible(checkboxDeliciousInbox.isSelected());
        previewPanel.setLinksVisible(checkboxRecent.isSelected());
        previewPanel.setShowPeople(radioSubscriptions.isSelected());
        previewPanel.updateCenter();
    }
    
    public void setStatus(String s) {
        if ((s != null) && (s.length() > 0))
                stepDetails[2] = s;
        stepText.setText(buildStatusString(currentStep));
    }
    
    private String buildStatusString(int step) {
        StringBuffer result = new StringBuffer();
        result.append("<html><head></head><body>");
        result.append("<b><u>Steps to be performed</u></b><br>");
        //result.append("<ul>");
        for (int i=0; i<STEP_DESCRIPTIONS.length; i++) {
            result.append("&nbsp;&nbsp;&nbsp;");
            if (step == i)
                result.append("<b>");
            result.append("&raquo;&nbsp;");
            result.append(STEP_DESCRIPTIONS[i]);
            if (step == i)
            	result.append("</b>");
            result.append("<br>");
            result.append("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
            result.append(stepDetails[i]);
            result.append("<br>");
            //result.append("</li>");
        }
        //result.append("</ul>");
        result.append("</body></html>");
        return result.toString();
    }
    
    private void startThread() {
        enableControls(false);
        Thread childThread = new Thread(this);
        childThread.start();
    }
    
    private void stopThread() {
        //TODO:
    }
    
    private void enableControls(boolean enabled) {
        this.okButton.setEnabled(enabled);
        this.cancelButton.setEnabled(!enabled);
        this.usernameText.setEnabled(enabled);
        this.passwordText.setEnabled(enabled);
        this.checkboxRecent.setEnabled(enabled);
        //Cursor theCursor = enabled ? Cursor.getDefaultCursor() : Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
        //Component[] components = this.getContentPane().getComponents();
        //for (int i=0; i<components.length; i++)
        //    components[i].setCursor(theCursor);
    }
    

    public void run() {
        /** Current state...useful for saying what happened during exception catch */
        String state = "initializing";
        /** Throttle...if this is > 0, only download a few entries (good for quick debugging) */
        int throttle = 0;
        /** Counter for number of things downloaded */
        int count = 0;
        /** The downloaded entries */
        Vector entries = new Vector();
        /** The bookmark DOM datastructure */
        Document doc = null;
        /** Are we subscribed to ourself? */
        boolean foundSelf = false;
        String username = this.usernameText.getText();
        resetStepDetails();
        try {
            throttle = Integer.parseInt(System.getProperty("tastysafari.throttle", "0"));
        }catch(Throwable t){}
        try{
            // #####
            this.currentStep = 0;
            setStatus("");
            doc = this.bookmarkEditor.load();
            // #####
            this.currentStep++;
            setStatus("");
            state = "constructing API object";
            DeliciousApi delicious = new DeliciousApi(usernameText.getText(), passwordText.getText(), urlText.getText());
            state = "downloading dates";
            String[] dates = new String[0];
            if (this.radioSubscriptions.isSelected())
            	dates = delicious.getInboxDates();
            // If throttled, grab only the last few dates
            if ((throttle > 0) && (dates.length > throttle)) {
                String[] tempDates = new String[throttle];
                for (int i=0; i<throttle; i++)
                    tempDates[i] = dates[dates.length - throttle + i];
                dates = tempDates;
            }
            stepDetails[currentStep] = "downloaded " + dates.length + " date" + (dates.length == 1 ? "" : "s");
            setStatus("");
            for (int i=0; i<dates.length; i++) {
                state = "downloading posts for date #" + i;
                PostEntry[] posts = delicious.getInbox(dates[i]);
                if (!foundSelf)
                	for (int j=0; j<posts.length; j++)
                        if (posts[j].getUser().equalsIgnoreCase(username))
                            foundSelf = true;
                entries.add(posts);
                count += posts.length;
            }
            // If not subscribed to self, get our own bookmarks
            stepDetails[currentStep] = "downloading posts for " + username;
            setStatus("");
            if (!foundSelf) {
                dates = delicious.getDates();
                for (int i=0; i<dates.length; i++){
                    PostEntry[] posts = delicious.getPosts(dates[i]);
                    entries.add(posts);
                    count += posts.length;
                }
            }
            // Get the most recent "x" posts
            PostEntry[] mostRecent = null;
            if (this.checkboxRecent.isSelected()) {
            	mostRecent = delicious.getInbox(null);
            	stepDetails[currentStep] = "Downloaded " + count + " post" + (count == 1 ? "" : "s") + ", " + mostRecent.length + " most recent";
            }else{
                stepDetails[currentStep] = "Downloaded " + count + " post" + (count == 1 ? "" : "s");
            }
            // #####
            this.currentStep++;
            setStatus("");
            SafariBookmarkMerger merger = new SafariBookmarkMerger();
            merger.setUsername(usernameText.getText());
            merger.merge(doc, entries, mostRecent, this,
                         checkboxDeliciousHome.isSelected(),
						 checkboxMyDelicious.isSelected(),
						 checkboxDeliciousInbox.isSelected());
            // #####
            this.currentStep++;
            setStatus("");
            bookmarkEditor.backupBookmarks();
            stepDetails[currentStep] = "There are currently " + bookmarkEditor.getBackupCount() + " previous backup" + (bookmarkEditor.getBackupCount() == 1 ? "" : "s");
            // #####
            this.currentStep++;
            setStatus("");
            bookmarkEditor.save(doc);
            File f = new File(bookmarkEditor.getBookmarkFilename());
            stepDetails[currentStep] = "" + f.length() + " bytes written";
            this.currentStep = -1;
            setStatus("");
        } catch(Throwable t) {
            t.printStackTrace();
            JOptionPane.showMessageDialog(this, "Problem " + state + ":\n" + t.toString(), "Unexpected Error", JOptionPane.ERROR_MESSAGE);
        }finally{
            enableControls(true);
            this.currentStep = -1;
            setStatus("");
        }
    }
    
    public void keyPressed(KeyEvent e) {
        int code = e.getKeyCode();
        int modifier = e.getModifiers();
        //System.out.println(e.getModifiers());
        //System.out.println(code);
        if (code == KeyEvent.VK_ESCAPE) {
            endProgram();
        } else if ((modifier == KeyEvent.META_MASK) && (code == KeyEvent.VK_W)) {
            endProgram();
        } else if(code == KeyEvent.VK_ENTER) {
            if (okButton.isEnabled())
                okButton.doClick();
        }
    }
    public void keyReleased(KeyEvent arg0) {}
    public void keyTyped(KeyEvent arg0) {}

    public class Actions implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			String source = event.getActionCommand();
            source = (source == null) ? "" : source;
            if (source.equals("okButton"))
                startThread();
            if (source.equals("cancelButton"))
                stopThread();
		}
    }
    
    public class HighlightFocuser implements FocusListener {
		public void focusGained(FocusEvent arg0) {
            Object object = arg0.getSource();
            if (object instanceof JTextField) {
                JTextField field = (JTextField) object;
                field.setSelectionStart(0);
                field.setSelectionEnd(field.getText().length());
            }
		}
		public void focusLost(FocusEvent arg0) {}
    }
        
    public static void main(String[] argv) {
        MainWindow window = new MainWindow();
        window.setVisible(true);
        window.usernameText.requestFocus();
    }
}
