//Copyright (C) 2004, Brian Enigma <enigma at netninja.com>
//This file is part of Tasty Safari.
//
//Tasty Safari is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//Tasty Safari is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with Foobar; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package org.ninjasoft.tastysafari.swingui;

import java.awt.*;
import javax.swing.*;

/**
 * Panel to display a preview of your Safari bookmark bar
 * @author enigma
 */
public class PreviewPanel extends JPanel {
    private static final ImageIcon iconTop = new ImageIcon(PreviewPanel.class.getResource("/org/ninjasoft/tastysafari/graphics/top.gif"));
    private static final ImageIcon iconLeft = new ImageIcon(PreviewPanel.class.getResource("/org/ninjasoft/tastysafari/graphics/left.gif"));
    private static final ImageIcon iconRight = new ImageIcon(PreviewPanel.class.getResource("/org/ninjasoft/tastysafari/graphics/right.gif"));
    private static final ImageIcon iconAdd = new ImageIcon(PreviewPanel.class.getResource("/org/ninjasoft/tastysafari/graphics/add.gif"));
    private static final ImageIcon iconHome = new ImageIcon(PreviewPanel.class.getResource("/org/ninjasoft/tastysafari/graphics/home.gif"));
    private static final ImageIcon iconMy = new ImageIcon(PreviewPanel.class.getResource("/org/ninjasoft/tastysafari/graphics/my.gif"));
    private static final ImageIcon iconInbox = new ImageIcon(PreviewPanel.class.getResource("/org/ninjasoft/tastysafari/graphics/inbox.gif"));
    private static final ImageIcon iconPeople = new ImageIcon(PreviewPanel.class.getResource("/org/ninjasoft/tastysafari/graphics/people.gif"));
    private static final ImageIcon iconCategories = new ImageIcon(PreviewPanel.class.getResource("/org/ninjasoft/tastysafari/graphics/categories.gif"));
    private static final ImageIcon iconLinks = new ImageIcon(PreviewPanel.class.getResource("/org/ninjasoft/tastysafari/graphics/links.gif"));
    private static final ImageIcon iconBlank = new ImageIcon(PreviewPanel.class.getResource("/org/ninjasoft/tastysafari/graphics/blank.gif"));
    
    private boolean showHome = true;
    private boolean showMy = true;
    private boolean showInbox = true;
    private boolean showPeople = true;
    private boolean showLinks = false;
    
    private JPanel centerPanel = new JPanel();
    
    public PreviewPanel() {
        this.setSize(iconLeft.getIconWidth() + iconTop.getIconWidth() + iconRight.getIconWidth(),
                     iconLeft.getIconHeight());
        this.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.gridx = c.gridy = 0;
        c.gridwidth = 3;
        c.gridheight = 1;
        c.anchor = GridBagConstraints.SOUTH;
        c.fill = GridBagConstraints.NONE;
        c.weightx = c.weighty = 0;
        c.insets = new Insets(0,0,0,0);
        c.ipadx = c.ipady = 0;
        this.add(new JLabel(iconTop), c);
        c.gridy++;
        c.gridwidth = 1;
        c.anchor = GridBagConstraints.NORTHEAST;
        this.add(new JLabel(iconLeft), c);
        c.gridx++;
        c.anchor = GridBagConstraints.NORTH;
        this.add(centerPanel, c);
        c.anchor = GridBagConstraints.NORTHWEST;
        c.gridx++;
        this.add(new JLabel(iconRight), c);
        centerPanel.setSize(iconTop.getIconWidth(), iconLeft.getIconHeight());
        centerPanel.setLayout(new GridBagLayout());
        updateCenter();
    }
    
    public void updateCenter() {
        int shown = 0;
        GridBagConstraints c = new GridBagConstraints();
        c.gridx = c.gridy = 0;
        c.gridwidth = c.gridheight = 1;
        c.weightx = c.weighty = 1;
        c.anchor = GridBagConstraints.NORTH;
        c.fill = GridBagConstraints.NONE;
        c.insets = new Insets(0,0,0,0);
        c.ipadx = c.ipady = 0;
        centerPanel.removeAll();
        System.out.println(centerPanel.getComponentCount());
        shown++;
        centerPanel.add(new JLabel(iconAdd), c);
        if (showHome) {
        	c.gridy++;
            shown++;
        	centerPanel.add(new JLabel(iconHome), c);
        }
        if (showMy) {
        	c.gridy++;
            shown++;
        	centerPanel.add(new JLabel(iconMy), c);
        }
        if (showInbox) {
        	c.gridy++;
            shown++;
            centerPanel.add(new JLabel(iconInbox), c);
        }
        if (showPeople) {
        	c.gridy++;
            shown += 3;
            centerPanel.add(new JLabel(iconPeople), c);
        }else{
            c.gridy++;
            shown += 3;
            centerPanel.add(new JLabel(iconCategories), c);
        }
        if (showLinks) {
            c.gridy++;
            shown += 3;
            centerPanel.add(new JLabel(iconLinks), c);
        }
        for (int i = shown; i<10; i++) {
            c.gridy++;
            centerPanel.add(new JLabel(iconBlank), c);
        }
        this.validate();
    }
    
    public void setHomeVisible(boolean b)           {this.showHome = b;}
    public void setMyVisible(boolean b)             {this.showMy = b;}
    public void setInboxVisible(boolean b)          {this.showInbox = b;}
    public void setLinksVisible(boolean b)          {this.showLinks = b;}
    public void setShowPeople(boolean b)            {this.showPeople = b;}
}
